#include <Servo.h>

// Pins 
const int ECHO_PIN = 6;
const int TRIG_PIN = 7; 
const int SERVO_PIN = 9;
const int LED1_PIN = 13;
const int LED2_PIN = 12; 

// defines variables
long duration;                                        // variable for the duration of sound wave travel
int distance;                                         // variable for the distance measurement
int pos = 0;                                          // variable for the rotations for loop counter
int iterations;                                       // variable for storing itteration count of switch_lights
bool once;                                            // makes sure the default position of the servo is set only once

// defines servo object
Servo my_servo;

void setup() {

  pinMode(LED1_PIN, OUTPUT);
  pinMode(LED2_PIN, OUTPUT);
  pinMode(TRIG_PIN, OUTPUT);                          // Sets the trigPin as an OUTPUT
  pinMode(ECHO_PIN, INPUT);                           // Sets the echoPin as an INPUT
  Serial.begin(9600);                                 // Serial Communication is starting with 9600 of baudrate speed
  Serial.println("Serial Communications Engaged");    // print some text in Serial Monitor
  
  my_servo.attach(SERVO_PIN);                         // Init servo pin
}

// returns a long with the current distance reading from the ultrasonic sensor
long calculate_distance(){
  digitalWrite(TRIG_PIN, LOW);                        // Clears the trigPin condition
  delayMicroseconds(2);

  digitalWrite(TRIG_PIN, HIGH);                       // Sets the trigPin HIGH (ACTIVE) for 10 microseconds
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);

  duration = pulseIn(ECHO_PIN, HIGH);                 // Reads the echoPin, returns the sound wave travel time in microseconds
  
  return duration * 0.034 / 2;                        // calculates distance - speed of sound wave divided by 2 
}


// allows us to set lights on or off (HIGH/LOW)
void set_lights(bool input){
  if(input){
    digitalWrite(LED1_PIN, HIGH);
    digitalWrite(LED2_PIN, HIGH);
  } else {
    digitalWrite(LED1_PIN, LOW);
    digitalWrite(LED2_PIN, LOW);
  }

}


void flip_switch(){ 

  // first checks if the itterations counter is even or odd (to check direction)
  // then rotates the servo using a for loop moving it 1 degree at a time
  if(iterations % 2 == 0){

    // for the on position the switch needs to first go backwards so it has time
    // to generate a little more momentum 
    for (pos = 90; pos >= 10; pos -= 1) {       // goes from 90 degrees to 10 degrees
    my_servo.write(pos);                        // tell servo to go to position in variable 'pos'
    delay(5);                                   // waits 5ms for the servo to reach the position
    }

    for (pos = 90; pos <= 210; pos += 1) {      // goes from 0 degrees to 180 degrees
    my_servo.write(pos);                        // tell servo to go to position in variable 'pos'
    delay(5);                                   // waits 5ms for the servo to reach the position
    }

  } else {

  	for (pos = 90; pos >= -40; pos -= 1) {      // goes from 90 degrees to -40 degrees
    my_servo.write(pos);                        // tell servo to go to position in variable 'pos'
    delay(15);                                  // waits 15ms for the servo to reach the position
    }

  }

  iterations++;                                // records itterations of loop (so we know to go forward or backward)

} 

void loop() {
  
  // grabs the distance reading from the sonar
  distance = calculate_distance();

  // Displays the distance on the Serial Monitor
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.println(" cm");
  
  // deturmines what to do based on current distance reading
  if(distance < 20){              // if less than 20cm - lights solid and flip switch
    once = true;
    set_lights(true);
    flip_switch();

  } else {                        // if 20cm+ longer blinking waits (conserves power)

    if(once){                     // sets default position to servo
      my_servo.write(90);
      once = false;
    }
    
    set_lights(true);             // default blinking pattern 
    delay(600);
    set_lights(false);
    delay(600);

  } 
  
}